#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "../include/oberon.h"

struct string_stack
{
	char * string;
	struct string_stack * next;
};

static oberon_context_t * ctx;
static oberon_module_t * mod;
static const char * module;
static const char * out_path;
static struct string_stack * path_list;

static void
add_to_stack(struct string_stack ** stack, char * string)
{
	struct string_stack * p = malloc(sizeof *p);
	p -> string = string;
	p -> next = *stack;
	*stack = p;
}

static FILE *
open_file(struct string_stack * path, const char * ext, const char * name, const char * mode)
{
	FILE * fp = NULL;
	while(fp == NULL && path != NULL)
	{
		char fname[256];
		snprintf(fname, 256, "%s/%s.%s", path -> string, name, ext);
		fp = fopen(fname, mode);
		path = path -> next;
	}
	return fp;
}

static const char *
import_module(const char * name)
{
	FILE * fp;
	fp = open_file(path_list, "obn", name, "r");
	if(fp == NULL)
	{
		return NULL;
	}

	char * source;
	size_t len;

	fseek(fp, 0, SEEK_END);
	len = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	source = calloc(1, len + 1);
	fread(source, len, 1, fp);

	return source;
}

static void
init(int argc, char ** argv)
{
	int i = 1;
	add_to_stack(&path_list, ".");
	out_path = ".";

	while(argc > 2)
	{
		if(argc > 2 && strcmp(argv[i], "-d") == 0)
		{
			out_path = argv[i + 1];
			argc -= 2;
			i += 2;
		}
		else if(argc > 2 && strcmp(argv[i], "-I") == 0)
		{
			add_to_stack(&path_list, argv[i + 1]);
			argc -= 2;
			i += 2;
		}
		else
		{
			break;
		}
	}

	if(argc != 2 || argv[i][0] == '-')
	{
		printf("usage: obn [-d out_dir] module\n");
		exit(1);
	}

	module = argv[i];
}

int
main(int argc, char ** argv)
{
	const char * code;

	init(argc, argv);

	code = import_module(module);
	if(code == NULL)
	{
		printf("can't open module source %s\n", module);
		return 0;
	}

	ctx = oberon_create_context(import_module);
	oberon_set_out_directory(ctx, out_path);

	mod = oberon_compile_module(ctx, code);

	oberon_destroy_context(ctx);

	return 0;
}
